@@ -1,6 +1,6 @@
 package Data::CloudWeights;
 
-# @(#)$Id: CloudWeights.pm 44 2008-02-28 11:08:43Z pjf $
+# @(#)$Id: CloudWeights.pm 48 2008-05-19 17:07:59Z pjf $
 # Originally WWW::CloudCreator. Now returns even more raw result
 
 use strict;
@@ -8,7 +8,7 @@ use warnings;
 use base qw(Class::Accessor::Fast);
 use Readonly;
 
-use version; our $VERSION = qv( sprintf '0.1.%d', q$Rev: 44 $ =~ /\d+/gmx );
+use version; our $VERSION = qv( sprintf '0.2.%d', q$Rev: 48 $ =~ /\d+/gmx );
 
 Readonly my %ATTRS =>
    ( # Input. Set in constructor or call mutator before formation method
@@ -18,8 +18,12 @@ Readonly my %ATTRS =>
                             CCFF66 FFFF33 FF6600 FF0000) ],
      decimal_places => 2,          # Defaults for ems
      hot_colour     => q(FF0000),  # Red
+     limit          => 0,          # Max size of returned list. Zero no limit
      max_size       => 2.0,        # Output size no more than
      min_size       => 0.66,       # Output size no less than
+     sort_field     => q(tag),     # Output sorted by this field
+     sort_order     => q(asc),     # Sort order - asc   or desc
+     sort_type      => q(alpha),   # Sort type  - alpha or numeric
 
      # Output. Calling accessors becomes useful after last call to add method
      max_count      => 0,          # Current max value across all tags cloud
@@ -28,9 +32,27 @@ Readonly my %ATTRS =>
 
      # Private.
      _base          => undef,
-     _counts_ref    => undef,
+     _indx          => undef,
      _step          => undef,
-     _values_ref    => undef );
+     _tags          => undef, );
+
+Readonly my %SORTS =>
+   ( alpha => {
+        asc  => sub {
+           my $f = shift; return sub { $_[ 0 ]->{ $f } cmp $_[ 1 ]->{ $f } }
+        },
+        desc => sub {
+           my $f = shift; return sub { $_[ 1 ]->{ $f } cmp $_[ 0 ]->{ $f } }
+        },
+     },
+     numeric => {
+        asc  => sub {
+           my $f = shift; return sub { $_[ 0 ]->{ $f } <=> $_[ 1 ]->{ $f } }
+        },
+        desc => sub {
+           my $f = shift; return sub { $_[ 1 ]->{ $f } <=> $_[ 0 ]->{ $f } }
+        },
+     }, );
 
 __PACKAGE__->mk_accessors( keys %ATTRS );
 
@@ -44,11 +66,10 @@ sub new {
       $self->$_( $args->{ $_ } );
    }
 
-   $self->_base(       [] );
-   $self->_counts_ref( {} );
-   $self->_step(       [] );
-   $self->_values_ref( {} );
-
+   $self->_base( [] );
+   $self->_indx( {} );
+   $self->_step( [] );
+   $self->_tags( [] );
    return $self;
 }
 
@@ -56,7 +77,7 @@ sub add {
    # Include the passed args in this cloud's formation
    my ($me, $tag, $count, $value) = @_;
 
-   return unless ($tag); # Mandatory arg used as a key in counts and values
+   return unless ($tag); # Mandatory arg used as a key in tag ref index
 
    # Mask out null strings and negative numbers from the passed count value
    $count  = defined $count ? abs $count : 0;
@@ -64,57 +85,81 @@ sub add {
    # Add this count to the total for this cloud
    $me->total_count( $me->total_count + $count );
 
-   # Store the count. Calls with the same tag are cumulative
-   $count += $me->_counts_ref->{ $tag } || 0;
-   $me->_counts_ref->{ $tag } = $count;
+   if (exists $me->_indx->{ $tag }) {
+      # Calls with the same tag are cumulative
+      $count += $me->_indx->{ $tag }->{count};
+      $me->_indx->{ $tag }->{count} = $count;
+
+      if (defined $value) {
+         my $tag_value = $me->_indx->{ $tag }->{value};
+
+         # Make an array if there are two or more calls to add the same tag
+         if ($tag_value && ref $tag_value ne q(ARRAY)) {
+            $me->_indx->{ $tag }->{value} = [ $tag_value ];
+         }
+
+         # Push passed value in each call onto the values array.
+         if ($tag_value) { push @{ $me->_indx->{ $tag }->{value} }, $value }
+         else { $me->_indx->{ $tag }->{value} = $value }
+      }
+   }
+   else {
+      # Create a new tag reference and add to both list and index
+      my $tag_ref = { count => $count, tag => $tag, value => $value };
+      $me->_indx->{ $tag } = $tag_ref;
+      push @{ $me->_tags }, $tag_ref;
+   }
 
    # Update this cloud's max and min values
    $me->max_count( $count ) if ($count > $me->max_count);
    $me->min_count( $count ) if ($me->min_count == -1);
    $me->min_count( $count ) if ($count < $me->min_count);
 
-   if (defined $value) {
-      my $tag_value = $me->_values_ref->{ $tag };
-
-      # Make an array if there are two or more calls to add the same tag
-      if ($tag_value && ref $tag_value ne q(ARRAY)) {
-         $me->_values_ref->{ $tag } = [ $tag_value ];
-      }
-
-      # Push passed value in each call onto the values array.
-      if ($tag_value) { push @{ $me->_values_ref->{ $tag } }, $value }
-      else { $me->_values_ref->{ $tag } = $value }
-   }
-
    # Return the current cumulative count for this tag
-   return $me->_counts_ref->{ $tag };
+   return $count;
 }
 
 sub formation {
    # Calculate the result set for this cloud
-   my ($count, $me, $ntags, $out, $prec, $ratio, $rng, $size, $step, @tags);
+   my ($count, $field, $me, $ntags, $orderby, $out, $prec, $ratio);
+   my ($rng, $size, $sort_ref, $step);
 
    $me    = shift;
    $prec  = 10**$me->decimal_places;
    $rng   = abs $me->max_count - $me->min_count || 1;
    $step  = ($me->max_size - $me->min_size) / $rng;
-   $ntags = @tags = keys %{ $me->_counts_ref };
+   $ntags = @{ $me->_tags };
    $out   = [];
 
    return $out if ($ntags == 0); # No calls to add were made
 
    if ($ntags == 1) {            # One call to add was made
       $out = [ { colour  => $me->hot_colour || pop @{ $me->colour_pallet },
-                 count   => $me->_counts_ref->{ $tags[0] },
+                 count   => $me->_tags->[0]->{count},
                  percent => 100,
                  size    => $me->max_size,
-                 tag     => $tags[0],
-                 value   => $me->_values_ref->{ $tags[0] } } ];
+                 tag     => $me->_tags->[0]->{tag},
+                 value   => $me->_tags->[0]->{value} } ];
       return $out;
    }
 
-   for (sort @tags) {            # Multiple calls to add were made
-      $count = $me->_counts_ref->{ $_ };
+   # Multiple calls to add were made, determine the sorting method
+   if ($field = $me->sort_field) {
+      if (ref $field) { $sort_ref = $field } # User supplied subroutine
+      else {
+         $orderby  = $SORTS{ lc $me->sort_type  }
+                           { lc $me->sort_order }->( $field );
+         # Protect against wrong sort type for the data
+         $sort_ref = $field ne q(tag)
+                   ? sub { return $orderby->( @_ )
+                               || $_[0]->{tag} cmp $_[1]->{tag} }
+                   : $orderby;
+      }
+   }
+   else { $sort_ref = sub { return 0 } } # No sorting if sort field is undef
+
+   for (sort { $sort_ref->( $a, $b ) } @{ $me->_tags }) {
+      $count = $_->{count};
       $ratio = $count / $me->total_count;
       $size  = $me->min_size + $step * ($count - $me->min_count);
 
@@ -124,8 +169,10 @@ sub formation {
                         count   => $count,
                         percent => (int 0.5 + $prec * 100 * $ratio) / $prec,
                         size    => (int 0.5 + $prec * $size) / $prec,
-                        tag     => $_,
-                        value   => $me->_values_ref->{ $_ } };
+                        tag     => $_->{tag},
+                        value   => $_->{value} };
+
+      last if ($me->limit && @{ $out } == $me->limit);
    }
 
    return $out;
@@ -136,7 +183,9 @@ sub formation {
 sub _arg_list {
    my ($me, @rest) = @_;
 
-   return $rest[0] && ref $rest[0] eq q(HASH) ? $rest[0] : { @rest };
+   return {} unless ($rest[0]);
+
+   return ref $rest[0] eq q(HASH) ? $rest[0] : { @rest };
 }
 
 sub _hex2dec {
@@ -176,7 +225,7 @@ sub _calculate_temperature {
       # Select colour from the pallet by allocating the value to a band
       $bands  = scalar @{ $me->colour_pallet };
       $index  = int 0.5 + ($cnt * ($bands - 1) / $rng);
-      $colour = $me->colour_pallet->[$index];
+      $colour = $me->colour_pallet->[ $index ];
    }
 
    return $colour;
@@ -194,7 +243,7 @@ Data::CloudWeights - Calculate values for an HTML tag cloud
 
 =head1 Version
 
-0.1.$Rev: 44 $
+0.2.$Rev: 48 $
 
 =head1 Synopsis
 
@@ -204,7 +253,7 @@ Data::CloudWeights - Calculate values for an HTML tag cloud
    my $cloud = Data::CloudWeights->new( \%cfg );
 
    # Add one or more tags to the cloud
-   $cloud->add( $name, $count, $value );
+   $cloud->add( $tag, $count, $value );
 
    # Calculate the tag cloud values
    my $nimbus = $cloud->formation();
@@ -254,6 +303,10 @@ changed tag font size can be set in pixies
 The six character hex colour for the highest count in the
 cloud. Defaults to FF0000 (red)
 
+=head3 limit
+
+Limits the size of the returned list. Defaults to zero, no limit
+
 =head3 max_size
 
 The upper boundary value to which the highest count in the cloud is
@@ -264,6 +317,23 @@ scaled. Defaults to 2.0 (ems)
 The lower boundary value to which the smallest count in the cloud is
 scaled. Defaults to 0.66 (ems)
 
+=head3 sort_field
+
+Select the field to sort the output by. Values are; I<tag>, I<count>
+or I<value>.  If set to I<undef> the output order will be the same as
+the order of the calls to C<add>. If set to a code ref it will be
+called as a sort comparison subroutine and passed two tag references
+whose keys are values listed above
+
+=head3 sort_order
+
+Either I<asc> for ascending or I<desc> for descending sort order
+
+=head3 sort_type
+
+Either I<alpha> to use the C<cmp> operator or I<numeric> to use the
+C<E<lt>=E<gt>> operator in sorting comparisons
+
 =head1 Subroutines/Methods
 
 =head2 add
@@ -339,6 +409,10 @@ None
 
 This did not let me calculate font sizes in ems
 
+=item L<HTML::TagCloud::Sortable>
+
+I lifted the sorting code from here
+
 =back
 
 =head1 Dependencies
@@ -367,7 +441,7 @@ Peter Flanigan, C<< <Support at RoxSoft.co.uk> >>
 
 =head1 License and Copyright
 
-Copyright (c) 2007 Peter Flanigan. All rights reserved.
+Copyright (c) 2008 Peter Flanigan. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself. See L<perlartistic>.
